import setuptools

setuptools.setup(
    name='expressmoney',
    version='0.21',
    description='Utility for ExpressMoney Company',
    packages=setuptools.find_packages(),
    author='Sergey Zherebtsov',
    author_email='sergey@expressmoney.com',
    url='http://expressmoney.com/',
    zip_safe=False,
    python_requires='>=3.8',
)
